<?php

/**
 * Home Block PushAlert Admin and FrontEnd
 *
 * @category Registration
 * @package  PushAlert
 * @author   Team PushAlert <support@pushalert.co>
 * @license  https://opensource.org/licenses/OSL-3.0 OSL Open Software License
 * @link     https://pushalert.co
 */

namespace PushAlert\PushNotifications\Block;

/**
 * Home Block
 *
 * @author   Team PushAlert <support@pushalert.co>
 * @license  https://opensource.org/licenses/OSL-3.0 OSL Open Software License
 * @link     https://pushalert.co
 */
class Home extends \Magento\Framework\View\Element\Template
{

    /**
     * Constructor
     *
     * @param \Magento\Backend\App\Action\Context                $context     context
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig get config json stored
     * @param \Magento\Backend\Model\Auth\Session                $authSession get current user
     */
    private $imageBuilder;
    private $productConfigHelper;
    private $jsonEncoder;
    private $customerSession;
    private $cookieHandler;
    private $request;
    private $priceHelper;

    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Framework\Json\EncoderInterface $jsonEncoder,
        \Magento\Framework\App\Request\Http $request,
        \Magento\Catalog\Model\Product $product,
        \Magento\Framework\Registry $registry,
        \Magento\Catalog\Block\Product\ImageBuilder $imageBuilder,
        \Magento\Framework\Pricing\Helper\Data $priceHelper,
        \Magento\ConfigurableProduct\Helper\Data $productConfigHelper,
        \Magento\Customer\Model\SessionFactory $customerSession,
        \PushAlert\PushNotifications\Cookie\CookieHandler $cookieHandler
    ) {
        $this->scopeConfig = $context->getScopeConfig();
        $this->_registry = $registry;
        $this->imageBuilder = $imageBuilder;
        $this->customerSession = $customerSession;
        $this->priceHelper = $priceHelper;
        $this->productConfigHelper = $productConfigHelper;
        $this->_product = $product;
        $this->request = $request;
        $this->jsonEncoder = $jsonEncoder;
        $this->cookieHandler = $cookieHandler;
        parent::__construct($context);
    }
    
    public function getCookieHandler()
    {
        return $this->cookieHandler;
    }

    public function getCustomerID()
    {
        try {
            $customer = $this->customerSession->create();
            $customer_id = $customer->getCustomer()->getId();
            if ($customer_id) {
                return $customer_id;
            }
            return 0;
        } catch (\Exception $e) {
            return 0;
        }
    }

    public function paEncrypt($num, $key)
    {
        $output = "";
        $numLength = strlen($num);
        for ($i = 0; $i < $numLength; $i++) {
            $output .= $key[substr($num, $i, 1)];
        }
        return $output;
    }

    public function getImage($product, $image_size='product_page_image_small')
    {
        if ($product != null) {
            return $this->imageBuilder->setProduct($product)
                            ->setImageId($image_size)
                            ->create();
        }
        return '';
    }

    public function getProduct()
    {
        return $this->_registry->registry('current_product');
    }

    public function setProduct()
    {
        $product = $this->getProduct();
        if ($product != null) {
            $this->_product->load($product->getId());
        }
    }

    public function getProductId()
    {
        return $this->_product->getId();
    }

    public function getJsonConfig()
    {
        if ($this->_product->getTypeId() == \Magento\ConfigurableProduct\Model\Product\Type\Configurable::TYPE_CODE) {
            $options = $this->productConfigHelper->getOptions(
                $this->_product,
                $this->_product->getTypeInstance()->getUsedProducts($this->_product)
            );
            if ($options != null) {
                $config = $options['index'];
                return $this->jsonEncoder->encode($config);
            } else {
                return 0;
            }
        } else {
            return 0;
        }
    }

    public function getCurrentProduct()
    {
        try {
            $file = "pushalertlog.txt";
            $this->setProduct();
            if ($this->request->getFullActionName() == 'catalog_product_view' && $this->_product != null) {
                $storeUrl = $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_WEB);
                
                //you are on the product page
                $product_name = $this->_product->getName();
                $product_type = ($this->_product->getTypeId() ==
                \Magento\ConfigurableProduct\Model\Product\Type\Configurable::TYPE_CODE) ? 'variable' : 'simple';
                $variants = [];

                if ($product_type == 'variable') {
                    $childProducts = $this->_product->getTypeInstance()->getUsedProducts($this->_product);
                    if (!empty($childProducts)) {
                        foreach ($childProducts as $variant) {
                            if ($variant != null) {
                                $addToCartUrl = $storeUrl.'pa-add-to-cart?product-id='.$variant->getId();
                                
                                $totalPrice = $variant->getFinalPrice();
                                $variants[] = [
                                    "variant_id" => $this->_product->isAvailable() ? $variant->getId() : 0,
                                    "title" => $product_name,
                                    "price" => $totalPrice,
                                    "price_formatted" => $this->priceHelper->currency($totalPrice, true, false),
                                    "image" => $this->getImage($variant, "product_page_image_medium")->getImageUrl(),
                                    "outofstock" => !($variant->isAvailable() && $this->_product->isAvailable()),
                                    "icon" => $this->getImage($variant, "cart_page_product_thumbnail")->getImageUrl(),
                                    "url" => $this->_product->getProductUrl(),
                                    "add_to_cart" => $addToCartUrl,
                                ];
                            }
                        }
                    }
                }
                
                
                $addToCartUrl = $storeUrl.'pa-add-to-cart?product-id='.$this->_product->getId();

                $product_data = [
                    "id" => $this->_product->getId(),
                    "variant_id" => $product_type == 'simple' ? 0 : $this->_product->getId(),
                    "variants" => $variants,
                    "title" => $product_name,
                    "price" => $this->_product->getFinalPrice(),
                    "price_formatted" => $this->priceHelper->currency($this->_product->getFinalPrice(), true, false),
                    "type" => $product_type,
                    "image" => $this->getImage($this->_product, "product_page_image_medium")->getImageUrl(),
                    "outofstock" => !$this->_product->isAvailable(),
                    "url" => $this->_product->getProductUrl(),
                    "add_to_cart" => $addToCartUrl,
                    "icon" => $this->getImage($this->_product, "cart_page_product_thumbnail")->getImageUrl(),
                ];
                return $product_data;
            }
            return 0;
        } catch (\Exception $e) {
            return array("error" => $e->getMessage());
        }
    }

    /**
     * Get PushAlert config
     *
     * @return $pushalertConfig
     */
    public function getConfig()
    {
        $path = "corepushalert/settings";
        $pushalert_config = $this->scopeConfig->getValue($path, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        return $pushalert_config;
    }
}
