<?php

namespace PushAlert\PushNotifications\Helper;

class PushAlertHelper extends \Magento\Framework\App\Helper\AbstractHelper
{
    private $customerSession;
    private $urlFramework;
    private $customerRepository;
    private $_curl;

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository,
        \Magento\Framework\Url $urlFramework,
        \Magento\Framework\HTTP\Client\Curl $curl
    ) {
        $this->customerSession = $customerSession;
        $this->urlFramework = $urlFramework;
        $this->customerRepository = $customerRepository;
        $this->_curl=$curl;
        parent::__construct($context);
    }

    public function getConfig($config_path)
    {
        return $this->scopeConfig->getValue(
            $config_path,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    public function getCustomerID()
    {
        try {
            $customer_id = $this->customerSession->getCustomer()->getId();
            if ($customer_id) {
                return $customer_id;
            }
            return 0;
        } catch (\Exception $e) {
            return 0;
        }
    }

    public function getCustomerFirstName()
    {
        try {
            $customer_id = $this->customerSession->getCustomer()->getId();
            $customer = $this->customerRepository->getById($customer_id);
            if ($customer->getId()) {
                $customer_name = $customer->getFirstname();
                return $customer_name;
            }
            return null;
        } catch (\Exception $e) {
            return null;
        }
    }

    public function isHTTPSEnabled()
    {
        $pushalert_config = $this->getConfig("corepushalert/settings");
        if ($pushalert_config['enable_push'] && $pushalert_config['https_option']) {
            return true;
        } else {
            return false;
        }
    }

    public function getUrl($urlType, $data)
    {
        return $this->urlFramework->getUrl($urlType, $data);
    }

    public function isEnabled()
    {
        $pushalert_config = $this->getConfig("corepushalert/settings");
        if ($pushalert_config['enable_push']) {
            return true;
        } else {
            return false;
        }
    }

    public function getRestAPIKey()
    {
        $pushalert_config = $this->getConfig("corepushalert/settings");
        if ($pushalert_config['rest_api_key'] != '') {
            return $pushalert_config['rest_api_key'];
        } else {
            return false;
        }
    }

    public function backgroundPost($url)
    {
        try {
            $apiKey = $this->getRestAPIKey();
            if (!$apiKey) {
                return false;
            }

            $parts = parse_url($url);
            $fp = fsockopen(
                "ssl://" . $parts['host'],
                isset($parts['port']) ? $parts['port'] : 443,
                $errno,
                $errstr,
                30
            );
            
            if (!$fp) {
                return false;
            } else {
                $out = "POST " . $parts['path'] . " HTTP/1.1\r\n";
                $out .= "Host: " . $parts['host'] . "\r\n";
                $out .= "User-Agent: custom\r\n";
                $out .= "Content-Type: application/x-www-form-urlencoded\r\n";
                $out .= "Authorization: api_key=" . $apiKey . "\r\n";
                $out .= "Content-Length: " . strlen($parts['query']) . "\r\n";
                $out .= "Connection: Close\r\n\r\n";
                if (isset($parts['query'])) {
                    $out .= $parts['query'];
                }
                fwrite($fp, $out);
                fclose($fp);
                return true;
            }
        } catch (\Exception $e) {
            return false;
        }
    }
    
    public function writeLog($logMessage)
    {
        $apiKey = $this->getRestAPIKey();
        if (!$apiKey) {
            return false;
        }
        
        $post_vars = ["log" => $logMessage];
        $curlUrl = "https://api.pushalert.co/rest/v1/customLog";
        
        /*$headers = [];
        $headers[] = "Authorization: api_key=" . $apiKey;

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, "https://api.pushalert.co/rest/v1/customLog");
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post_vars));
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        //curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);

        $result = curl_exec($ch);
        $result = json_decode($result, true);*/
        $this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
        $this->_curl->addHeader("Authorization", "api_key=".$apiKey);
        $this->_curl->post($curlUrl, $post_vars);
        $response = json_decode($this->_curl->getBody(), true);
    }

    public function paSubscriptionCheck()
    {
        try {
            $cookie_id = $this->getConfig("corepushalert/settings/_pushalert_cookie_id");
            $pushalert_subs_id = filter_input(INPUT_COOKIE, 'pushalert_' . $cookie_id . 'subs_id');
            $pushalert_subs_status = filter_input(INPUT_COOKIE, 'pushalert_' . $cookie_id . 'subs_status');
            return (
                (isset($pushalert_subs_status) && $pushalert_subs_status == "subscribed") &&
                (isset($pushalert_subs_id) && $pushalert_subs_id != "")
            );
        } catch (\Exception $e) {
            return false;
        }
    }

    public function pushAlertEnableEcommerce()
    {
        try {
            $apiKey = $this->getRestAPIKey();
            if (!$apiKey) {
                return false;
            }

            $curlUrl = "https://api.pushalert.co/rest/v1/enableEcommerce";

            //POST variables
            $post_vars =[
                "type" => "magento"
            ];

            /*$headers = [];
            $headers[] = "Authorization: api_key=" . $apiKey;

            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $curlUrl);
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post_vars));
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

            $result = curl_exec($ch);*/
            $this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
            $this->_curl->addHeader("Authorization", "api_key=".$apiKey);
            $this->_curl->post($curlUrl, $post_vars);
            $response = json_decode($this->_curl->getBody(), true);
        } catch (\Exception $e) {
            return false;
        }
    }
}
