<?php

namespace PushAlert\PushNotifications\Observer;

use Magento\Framework\Event\ObserverInterface;
use PushAlert\PushNotifications\Helper\PushAlertHelper;
use Magento\Framework\App\Config\Storage\WriterInterface;

/**
 * Description of AbandonedCartObserver
 *
 * @author Abhinav
 */
class ConfigObserver implements ObserverInterface
{
    private $configWriter;
    private $_curl;
    private $helper;

    public function __construct(
        PushAlertHelper $helper,
        WriterInterface $configWriter,
        \Magento\Framework\HTTP\Client\Curl $curl
    ) {
        $this->helper = $helper;
        $this->configWriter = $configWriter;
        $this->_curl=$curl;
    }

    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        try {
            $this->pushAlertEnableEcommerce();
            $this->pushalertLoadSettings();
        } catch (\Exception $e) {
            return false;
        }
    }

    private function saveConfig($key, $value)
    {
        try {
            $this->configWriter->save("corepushalert/settings/" . $key, $value);
        } catch (\Exception $e) {
            return false;
        }
    }

    private function pushalertLoadSettings()
    {
        try {
            $apiKey = $this->helper->getRestAPIKey();
            if (!$apiKey) {
                return false;
            }

            $curlUrl = "https://api.pushalert.co/rest/v1/settings";

            //POST variables
            $post_vars = [];

            /*$headers = [];
            $headers[] = "Authorization: api_key=" . $apiKey;

            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $curlUrl);
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post_vars));
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

            $result = curl_exec($ch);
            $result = json_decode($result, true);*/
            $this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
            $this->_curl->addHeader("Authorization", "api_key=".$apiKey);
            $this->_curl->post($curlUrl, $post_vars);
            $result = json_decode($this->_curl->getBody(), true);

            if ($result['success']) {
                foreach ($result['data'] as $key => $value) {
                    $this->saveConfig($key, $value);
                }
            } else {
                return false;
            }
        } catch (\Exception $e) {
            return false;
        }
    }

    private function pushAlertEnableEcommerce()
    {
        try {
            $apiKey = $this->helper->getRestAPIKey();
            if (!$apiKey) {
                return false;
            }

            $curlUrl = "https://api.pushalert.co/rest/v1/enableEcommerce";

            //POST variables
            $post_vars = [
                "type" => "magento"
            ];

            /*$headers = [];
            $headers[] = "Authorization: api_key=" . $apiKey;

            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $curlUrl);
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post_vars));
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

            $result = curl_exec($ch);*/
            $this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
            $this->_curl->addHeader("Authorization", "api_key=".$apiKey);
            $this->_curl->post($curlUrl, $post_vars);
            $response = json_decode($this->_curl->getBody(), true);
        } catch (\Exception $e) {
            return false;
        }
    }
}
