<?php

namespace PushAlert\PushNotifications\Observer;

use Magento\Framework\Event\ObserverInterface;
use PushAlert\PushNotifications\Helper\PushAlertHelper;

/**
 * Description of AbandonedCartObserver
 *
 * @author abhin
 */
class PriceStockObserver implements ObserverInterface
{
    private $product;
    private $priceHelper;
    private $imageBuilder;
    private $configurableProduct;
    private $urlFramework;
    private $storeManager;
    private $_curl;
    private $helper;

    public function __construct(
        PushAlertHelper $helper,
        \Magento\Framework\Url $urlFramework,
        \Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable $configurableProduct,
        \Magento\Catalog\Model\Product $product,
        \Magento\Framework\Pricing\Helper\Data $priceHelper,
        \Magento\Catalog\Block\Product\ImageBuilder $imageBuilder,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\HTTP\Client\Curl $curl
    ) {
        $this->helper = $helper;
        $this->product = $product;
        $this->priceHelper = $priceHelper;
        $this->imageBuilder = $imageBuilder;
        $this->configurableProduct = $configurableProduct;
        $this->urlFramework = $urlFramework;
        $this->storeManager = $storeManager;
        $this->_curl=$curl;
    }

    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        $isEnabled = $this->helper->isEnabled();
        $priceDropEnabled = $this->helper->getConfig("corepushalert/settings/_pushalert_price_drop");
        if ($isEnabled && $priceDropEnabled) {
            $this->checkPriceChange($observer);
        }
    }

    private function getImage($product)
    {
        if ($product != null) {
            return $this->imageBuilder->setProduct($product)
                            ->setImageId("product_thumbnail_image")
                            ->create();
        }
        return '';
    }

    private function checkPriceChange(\Magento\Framework\Event\Observer $observer)
    {
        try {
            $event = $observer->getEvent();
            $updated_product = $event->getProduct();
            $parentId = $this->configurableProduct->getParentIdsByChild($updated_product->getId());
            $main_parent_id = 0;
            if (isset($parentId[0])) {
                $main_parent_id = $parentId[0];
                $this->product->load($parentId[0]);
            } else {
                $main_parent_id = $updated_product->getId();
                $this->product->load($updated_product->getId());
            }

            $changed_product_name = $this->product->getName();
            $product_type = ($this->product->getTypeId() ==
                \Magento\ConfigurableProduct\Model\Product\Type\Configurable::TYPE_CODE) ? 'variable' : 'simple';
            $sentProductId = $product_type=='simple'?$this->product->getId():$updated_product->getId();
            
            if($main_parent_id != $updated_product->getId()){
                $this->product->load($updated_product->getId());
            }
            
            $stock_check = $updated_product->getData('quantity_and_stock_status');

            $changed_product_url = $this->urlFramework->getUrl(
                'catalog/product/view',
                ['id' => $main_parent_id,
                '_nosid' => true,
                '_query' => ['___store' => 'default']]
            );

            //$original_price = $this->product->getData('price');
            //$current_price = $updated_product->getData('price');
            $original_price = $this->product->getFinalPrice();
            $current_price = $updated_product->getFinalPrice();
            

            //$myfile = fopen("newfile.txt", "w");
            //$txt = print_r($stock_check, true);
            //fwrite($myfile, $txt);
            //$txt = print_r($updated_product->getData('stock_data'), true);
            //fwrite($myfile, $txt);

            
            $quantity = $updated_product->getExtensionAttributes()->getStockItem()->getQty();
            // Get product stock status
            
            $new_stock_status = array(
                "is_in_stock" => $stock_check['is_in_stock'],
                "qty" => isset($stock_check['qty'])?$stock_check['qty']:$quantity
            );

            $storeUrl = $this->storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_WEB);
            $addToCartUrl = $storeUrl.'pa-add-to-cart?product-id='.$sentProductId;
            if ($original_price > $current_price) {
                $product_info = [
                    'id' => $main_parent_id,
                    'variant_id' => $product_type=='simple'?0:$updated_product->getId(),
                    'title' => $changed_product_name,
                    'price' => $current_price,
                    'price_formatted' => $this->priceHelper->currency($current_price, true, false),
                    'type' => $product_type,
                    'image' => $this->getImage($updated_product)->getImageUrl(),
                    'outofstock' => 0,
                    'url' => $changed_product_url,
                    'add_to_cart' => $addToCartUrl,
                    'changed' => 'price',
                    'currency' => ''
                ];
                $this->pushAlertProductUpdate($product_info);
            }

            $this->product->load($updated_product->getId());
            $old_stock_status = $this->product->getData('quantity_and_stock_status');
            $product_info="";
            $sentProductId = $product_type=='simple'?$this->product->getId():$updated_product->getId();
            $addToCartUrl = $storeUrl.'pa-add-to-cart?product-id='.$sentProductId;
            $txt = print_r($this->product->getData('quantity_and_stock_status'), true);
            //fwrite($myfile, (($old_stock_status['is_in_stock'] == false || $old_stock_status['is_in_stock'] == null || (int)$old_stock_status['qty'] == 0) && ($new_stock_status['is_in_stock'] == true && (int)$new_stock_status['qty'] > 0)).$txt);

            if (($old_stock_status['is_in_stock'] == false || $old_stock_status['is_in_stock'] == null || (int)$old_stock_status['qty'] == 0)
                && ($new_stock_status['is_in_stock'] == true && (int)$new_stock_status['qty'] > 0)) {
                $product_info = [
                    'id' => $main_parent_id,
                    'variant_id' => $product_type=='simple'?0:$updated_product->getId(),
                    'title' => $changed_product_name,
                    'price' => $current_price,
                    'price_formatted' => $this->priceHelper->currency($current_price, true, false),
                    'type' => $product_type,
                    'image' => $this->getImage($updated_product)->getImageUrl(),
                    'outofstock' => $new_stock_status['is_in_stock'] ? 0 : 1,
                    'url' => $changed_product_url,
                    'add_to_cart' => $addToCartUrl,
                    'changed' => 'outofstock',
                    'currency' => ''
                ];
                //fwrite($myfile, print_r($product_info, true));
                $this->pushAlertProductUpdate($product_info);
            }

            //fclose($myfile);
        } catch (\Exception $e) {
            return false;
        }
    }

    private function pushAlertProductUpdate($product_info)
    {
        try {
            $apiKey = $this->helper->getRestAPIKey();
            if (!$apiKey) {
                return false;
            }

            $curlUrl = "https://api.pushalert.co/rest/v1/product/update";

            $post_vars = [
                "product_info" => json_encode($product_info)
            ];

            if (!$this->helper->backgroundPost($curlUrl . "?" . http_build_query($post_vars))) {
                /*$headers = [];
                $headers[] = "Authorization: api_key=" . $apiKey;

                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $curlUrl);
                curl_setopt($ch, CURLOPT_POST, true);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post_vars));
                curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

                $result = curl_exec($ch);*/
                $this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
                $this->_curl->addHeader("Authorization", "api_key=".$apiKey);
                $this->_curl->post($curlUrl, $post_vars);
                $response = json_decode($this->_curl->getBody(), true);
            }
        } catch (\Exception $e) {
            return false;
        }
    }
}
