<?php

namespace PushAlert\PushNotifications\Observer;

use Magento\Framework\Event\ObserverInterface;
use PushAlert\PushNotifications\Helper\PushAlertHelper;

/**
 * Description of AbandonedCartObserver
 *
 * @author Abhinav
 */
class ShipmentObserver implements ObserverInterface
{
    private $urlFramework;
    private $_curl;
    private $helper;

    public function __construct(
        PushAlertHelper $helper,
        \Magento\Framework\Url $urlFramework,
        \Magento\Framework\HTTP\Client\Curl $curl
    ) {
        $this->helper = $helper;
        $this->urlFramework = $urlFramework;
        $this->_curl=$curl;
    }

    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        $isEnabled = $this->helper->isEnabled();
        $shipmentAlertEnabled = $this->helper->getConfig("corepushalert/settings/_pushalert_shipment_alert");
        if ($isEnabled && $shipmentAlertEnabled) {
            $this->checkOrderShipped($observer);
        }
    }

    private function checkOrderShipped(\Magento\Framework\Event\Observer $observer)
    {
        try {
            $shipment = $observer->getEvent()->getShipment();
            $order = $shipment->getOrder();
            $customer_id = $order->getCustomerId() != null ? $order->getCustomerId() : 0;
            $order_status_url = $this->urlFramework->getUrl(
                'sales/order/view',
                ['order_id' => $order->getId(),
                '_nosid' => true,
                '_query' => ['___store' => 'default']]
            );

            $tracking_url = $this->urlFramework->getUrl(
                'sales/order/shipment',
                ['order_id' => $order->getId(),
                '_nosid' => true,
                '_query' => ['___store' => 'default']]
            );

            if ($customer_id != 0) {
                $order_status_update = [
                    "order_id" => $order->getIncrementId(),
                    "order_status" => 'shipped',
                    "customer_id" => $customer_id,
                    "first_name" => $order->getCustomerFirstname(),
                    "last_name" => $order->getCustomerLastname(),
                    "order_status_url" => $order_status_url,
                    "tracking_url" => $tracking_url
                ];
                $this->pushalertTrackOrderShipment($order_status_update);
            }
        } catch (\Exception $e) {
            return false;
        }
    }

    private function pushalertTrackOrderShipment($order_info)
    {
        try {
            $apiKey = $this->helper->getRestAPIKey();
            if (!$apiKey) {
                return false;
            }

            $curlUrl = "https://api.pushalert.co/rest/v1/order/track";

            //POST variables
            $post_vars = [
                "order_info" => json_encode($order_info)
            ];

            if (!$this->helper->backgroundPost($curlUrl . "?" . http_build_query($post_vars))) {
                /*$headers = [];
                $headers[] = "Authorization: api_key=" . $apiKey;

                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $curlUrl);
                curl_setopt($ch, CURLOPT_POST, true);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post_vars));
                curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

                $result = curl_exec($ch);*/
                $this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
                $this->_curl->addHeader("Authorization", "api_key=".$apiKey);
                $this->_curl->post($curlUrl, $post_vars);
                $response = json_decode($this->_curl->getBody(), true);
            }
        } catch (\Exception $e) {
            return false;
        }
    }
}
